%%%%%%%%%%%%% Diagnostics script for the Japanese Vowel task %%%%%%%%%

% This script cannot be run on its own. It is called from the main script
% of this package, JapVow_MainScript, and needs many variables assigned in
% the main script.

% created and copyright: Herbert Jaeger, Feb 23, 2007. For terms of use,
% see JapVow_MainScript

%%%%% Declare for which sizes of voter's sets the diagnostics is computed.

% Example: if the main script was run with 500 experts, the setting 
%                voteSets = [1 5 10 20 50 100 200 500];
% will compute diagnostics for (a) 500 singleton voter sets, (b) 100 voter
% sets obtained by partitioning the 500 experts into 100 sets of size 5
% each, etc., up to (h) one voter set comprising all voters. The entries in 
% "voteSets" must all be divisors of the number of experts used in the main script
% (here: 500)
voteSets = [1 5 10 20 50 100 200];
nVoteSets = length(voteSets);
startFigNr = 3; % this script will generate numerous figure windows with 
                % successive figure numbers, starting with startFigNr


% reduce the training and testing teacher output collections in size by collapsing
% them in their temporal dimension (teacher outputs are constant over time,
% so keeping only the first time step teacher values loses no information).
allTrainOutsShort = zeros(270, 9);            
for i = 1:270
    allTrainOutsShort(i, :) = trainOutputs{i,1}(1, :);                
end
allTestOutsShort = zeros(370, 9);            
for i = 1:370                
    allTestOutsShort(i, :) = testOutputs{i,1}(1, :);               
end

% allocate collectors for the diagnostic results
meanResultsNRMSE_train = zeros(nVoteSets,1);
meanResultsClass_train = zeros(nVoteSets,1);
resultsNRMSE_train = cell(nVoteSets,1);
resultsClass_train = cell(nVoteSets,1);
meanResultsNRMSE_test = zeros(nVoteSets,1);
meanResultsClass_test = zeros(nVoteSets,1);
resultsNRMSE_test = cell(nVoteSets,1);
resultsClass_test = cell(nVoteSets,1);

% Carry out diagnostics for every vote set size
diagCounter = 0;
figCounter = 0;
for voteSet = voteSets
    diagCounter  = diagCounter + 1;
    
    resultsNRMSE_train{diagCounter,1} = zeros(nExperts/voteSet,1);
    resultsClass_train{diagCounter,1} = zeros(nExperts/voteSet,1);
    resultsNRMSE_test{diagCounter,1} = zeros(nExperts/voteSet,1);
    resultsClass_test{diagCounter,1} = zeros(nExperts/voteSet,1);
    for l = 1:nExperts/voteSet
        lumpedTrainOuts = ...
            mean(trainOutsIndividual(:,:, (l-1)*voteSet+1 : l*voteSet),3);
        lumpedTestOuts = ...
            mean(testOutsIndividual(:,:, (l-1)*voteSet+1 : l*voteSet),3);
        resultsNRMSE_train{diagCounter,1}(l,1) = ...
            mean(compute_error(lumpedTrainOuts, allTrainOutsShort)); 
        resultsNRMSE_test{diagCounter,1}(l,1) = ...
            mean(compute_error(lumpedTestOuts, allTestOutsShort)); 
        
        trainClassifications = zeros(270,9);
        testClassifications = zeros(370, 9);
        for i = 1:270    
            [maxVal, choice] = max(lumpedTrainOuts(i,:));
            trainClassifications(i,choice) = 1;    
        end
        for i = 1:370    
            [maxVal, choice] = max(lumpedTestOuts(i,:));
            testClassifications(i,choice) = 1;  
        end        
        
        trainMisclassifications = sum(sum(abs(trainClassifications - allTrainOutsShort)))/2;
        testMisclassifications = sum(sum(abs(testClassifications - allTestOutsShort)))/2;
        resultsClass_train{diagCounter,1}(l,1) = trainMisclassifications;
        resultsClass_test{diagCounter,1}(l,1) = testMisclassifications;
        %% for the first set of voters of a given size, plot the averaged
        %% outputs of the 9 voting outputs, vs. the training and testing
        %% cases, respectively
        if l == 1
            figure(startFigNr + figCounter); clf;
            plot(lumpedTrainOuts);
            title(sprintf('train outs, %g voters', voteSet));
            figCounter = figCounter + 1;
            
            figure(startFigNr + figCounter); clf;
            plot(lumpedTestOuts);
            title(sprintf('test outs, %g voters', voteSet));
            figCounter = figCounter + 1;
        end
    end
    meanResultsNRMSE_train(diagCounter,1) = mean(resultsNRMSE_train{diagCounter,1});
    meanResultsClass_train(diagCounter,1) = mean(resultsClass_train{diagCounter,1});
    meanResultsNRMSE_test(diagCounter,1) = mean(resultsNRMSE_test{diagCounter,1});
    meanResultsClass_test(diagCounter,1) = mean(resultsClass_test{diagCounter,1});
    
end

disp(sprintf('train NRMSEs   %s', num2str(meanResultsNRMSE_train')));
disp(sprintf('test NRMSEs   %s', num2str(meanResultsNRMSE_test')));
disp(sprintf('train MisClass   %s', num2str(meanResultsClass_train')));
disp(sprintf('test MisClass   %s', num2str(meanResultsClass_test')));

figure(startFigNr + figCounter); clf;
plot(voteSets, meanResultsClass_test');
title('misclass. vs. vote set size');
figCounter = figCounter + 1;

figure(startFigNr + figCounter); clf;
plot(voteSets, meanResultsNRMSE_test');
title('average NRMSE vs. vote set size');
figCounter = figCounter + 1;

