function outputSequence = test_esn_JapVow(inputSequence, esn)
% A specialized version of ESN testing for the situation in the Japanese 
% Vowels experiment. To be used in the context of the script
% "JapVow_MainScript" in the demo package associated with the paper: Herbert Jaeger, Mantas
% Lukosevicius, Dan Popovici, Optimization and Applications of Echo State
% Networks with Leaky Integrator Neurons. Neural Networks, April 2007 
% (preprint  available from first author's publications page)

% Adapted from test_esn (Version 1.0, April 30, 2006) June 30, 2006, H. Jaeger. 
% For terms of use, see JapVow_MainScript.

% input args:  
% inputSequence: is a nTrainingPoints * nInputUnits matrix that contains 
% the input we will run the esn on
% esn: the trained ESN structure 
% nForgetPoints: nr of initial time points to be discarded
%
% ouput:
% outputSequence is an array of size (size(inputSequence, 1)-nForgetPoints)
% x esn.nOutputUnits


esnTest = esn;

if esnTest.trained == 0
    error('The ESN is not trained. esn.trained = 1 for a trained network') ; 
end

% pretend that this esnTest uses only ordinary states to feed outputs (and
% not the compound states made from concatenating the states at the support
% points). Needed to make the subsequent "compute_statematrix" work.
esnTest.outputWeights = zeros(esnTest.nOutputUnits, esnTest.nInputUnits + esnTest.nInternalUnits);
stateCollectionFull = compute_statematrix(inputSequence, [], esnTest, 0) ; 
l = size(inputSequence,1);

stateCollection = zeros(1,esn.suppNr * (esnTest.nInputUnits + esnTest.nInternalUnits));

for s = 1:esnTest.suppNr
    % the selection of suppNr many support states, using interpolation if
    % necessary
    if s < esnTest.suppNr
        lowerInd = floor(s * (l-1)/esnTest.suppNr) + 1;
        a = s * (l-1)/esnTest.suppNr - floor(s * (l-1)/esnTest.suppNr);
        addState = (1 - a) * stateCollectionFull(lowerInd,:) + ...
            a * stateCollectionFull(lowerInd + 1,:) ;
    else
        addState = stateCollectionFull(end,:);
    end
    stateCollection(1, ...
                    (s-1)*(esnTest.nInputUnits + esnTest.nInternalUnits)+1:...
                    s * (esnTest.nInputUnits + esnTest.nInternalUnits),:) = addState;
    
end    

outputSequence = stateCollection * esn.outputWeights';
outputSequence = feval(esnTest.outputActivationFunction, outputSequence); 

%%%% scale and shift the outputSequence back to its original size

outputSequence = outputSequence - esnTest.teacherShift' ; 

outputSequence = outputSequence ./ esnTest.teacherScaling' ; 


