function internalState = leaky_esn(totalstate , esn , varargin )
% Update internal state using the leaky integrator neuron model.
%
% input arguments:
% totalstate: the previous totalstate, vector of size 
%     (esn.nInternalUnits + esn.nInputUnits + esn.nOutputUnits) x 1
% esn: the ESN structure
%
% output: 
% internalState: the updated internal state, size esn.nInternalUnits x 1
%
% Version 1, June 7, 2006, H.Jaeger

    previousInternalState = totalstate(1:esn.nInternalUnits, 1);
    internalState = (1 -  esn.leakage * esn.timeConstant) * previousInternalState + esn.timeConstant * ...
        feval(esn.reservoirActivationFunction ,...
        [ esn.internalWeights, esn.inputWeights, esn.feedbackWeights] * totalstate) ; 
    
    %%%% add noise to the Esn 
internalState = internalState + esn.noiseLevel * (rand(esn.nInternalUnits,1) - 0.5) ; 