function outputSequence = test_esn(inputSequence, esn, nForgetPoints)
% test_esn runs a trained ESN on a particular inputSequence

% input args:  
% inputSequence: is a nTrainingPoints * nInputUnits matrix that contains 
% the input we will run the esn on
% esn: the trained ESN structure 
% nForgetPoints: nr of initial time points to be discarded
%
% ouput:
% outputSequence is an array of size (size(inputSequence, 1)-nForgetPoints)
% x esn.nOutputUnits


%
% Version 1.0, April 30, 2006
% (c) Dan Popovici
% revision 1, June 6, 2006, H. Jaeger

if esn.trained == 0
    error('The ESN is not trained. esn.trained = 1 for a trained network') ; 
end

 
stateCollection = compute_statematrix(inputSequence, [], esn, nForgetPoints) ; 

outputSequence = stateCollection * esn.outputWeights' ; 

%%%% scale and shift the outputSequence back to its original size

nOutputPoints = length(outputSequence(:,1)) ; 

outputSequence = feval(esn.outputActivationFunction, outputSequence); 

outputSequence = outputSequence - repmat(esn.teacherShift',[nOutputPoints 1]) ; 

outputSequence = outputSequence / diag(esn.teacherScaling) ; 

  
  