function internalState = twi_esn(totalstate , esn , normDiff)
% Update internal state according to the time-warping invariant model
%
% input arguments:
% totalstate: the previous totalstate, vector of size 
%     (esn.nInternalUnits + esn.nInputUnits + esn.nOutputUnits) x 1
% esn: the ESN structure
% normDiff: the diff
%
% output: 
% internalState: the updated internal state, size esn.nInternalUnits x 1
%
% Version 1.0, June 7, 2006, H.Jaeger
   
previousInternalState = totalstate(1:esn.nInternalUnits, 1);
internalState = (1 - normDiff * esn.leakage * esn.timeConstant / esn.avDist) * ...
    previousInternalState + (normDiff * esn.timeConstant / esn.avDist) * ...
        feval(esn.reservoirActivationFunction ,...
        [ esn.internalWeights, esn.inputWeights, esn.feedbackWeights] * totalstate) ; 
    
    %%%% add noise to the Esn 
internalState = internalState + esn.noiseLevel * (rand(esn.nInternalUnits,1) - 0.5) ; 